/**
 *
 * @author zuojj (zuojj.com@gmail.com)
 * @description 兼容IE9 + 及现代浏览器
 * 2017-02-07 15:34:57
 */

(function (window) {
    function SogouTranslator(options) {
        var me = this,
            defaults;

        if (!(me instanceof SogouTranslator)) {
            return new SogouTranslator(options);
        }

        /**
         * @property {String}        url             翻译请求url
         * @property {String}        from            原语言
         * @property {String}        to              目标语言
         * @property {Number}        maxTextLength   单次请求最大纯文本长度(10 - 3000)
         * @property {Function}      ajax            ajax函数，可自定义或引用第三方库
         * @property {Function}      beforeTranslate 翻译开始前回调
         * @property {Function}      success         ajax成功时回调
         * @property {Function}      error           ajax失败时回调
         * @property {Function}      complete        ajax请求完成时回调
         * @property {Function}      appCallback     appCallback回调
         */
        defaults = me.defaults = {
            url: '',
            from: '',
            to: '',
            maxTextLength: 1000,
            ajax: '',
            beforeTranslate: '',
            success: '',
            error: '',
            onComplete: '',
            appCallback: ''
        };

        options = me.options = options || {};
        for (var key in defaults) {
            'undefined' === typeof options[key] && (options[key] = defaults[key]);
        }
    }

    SogouTranslator.prototype = {
        /**
         * [init description]
         * @return {[type]} [description]
         */
        init: function () {
            document.documentElement.style.display = 'block !important';
            document.body.style.display = 'block !important';

            this.getNodeText(document);
        },

        /**
         * [rewriteLink 重写A链接]
         * @param  {[type]} node [description]
         * @return {[type]}      [description]
         */
        rewriteLink: function (node) {
            node.setAttribute('target', '_top');
        },

        /**
         * [rewriteForm 重写Form]
         * @param  {[type]} node [description]
         * @return {[type]}      [description]
         */
        rewriteForm: function (node) {
            node.setAttribute('target', '_top');
        },

        /**
         * [uuid 获取uuid]
         * @return {[type]} [description]
         */
        uuid: function () {
            var uuid = '',
                i, random;
            for (i = 0; i < 32; i++) {
                random = Math.random() * 16 | 0;
                if (i === 8 || i === 12 || i === 16 || i === 20) uuid += '-'
                uuid += (i === 12 ? 4 : (i === 16 ? (random & 3 | 8) : random)).toString(16);
            }
            return uuid;
        },

        /**
         * [extendAttrName description]
         * @param  {[type]} attrName [description]
         * @return {[type]}          [description]
         */
        extendAttrName: function (attrName) {
            return 'data-trans-' + attrName;
        },

        /**
         * [extendAttrValue description]
         * @param  {[type]} plen [description]
         * @param  {[type]} slen [description]
         * @return {[type]}      [description]
         */
        extendAttrValue: function (id) {
            return id;
        },

        /**
         * [getNodeText 递归获取节点文本]
         * @return {[type]} [description]
         */
        getNodeText: function (doc) {
            var me = this,
                doc = doc || document,
                opts = me.options,
                maxTextLength = opts.maxTextLength,
                texts = [],
                items = [],
                strLen = '';

            me.getTextStartTime = Date.now();

            /* node节点list */
            var typeList = {
                getid: function (isSetAttr) {
                    return [texts.length, items.length + (isSetAttr ? -1 : 0)].join('-');
                },

                /**
                 * [setAttr description]
                 * @param {[type]} node [description]
                 * @param {[type]} type [description]
                 */
                setAttr: function (node, attrName) {
                    node.setAttribute(me.extendAttrName(attrName), me.extendAttrValue(this.getid(true)));
                    node.lang = opts.to;
                },

                /**
                 * [push item push]
                 * @param  {[type]} text [description]
                 * @param  {[type]} type [description]
                 * @return {[type]}      [description]
                 */
                push: function (text, type) {
                    if (!text && !text.trim()) return;
                    /*
                     * 由于后端模型支持问题，为解决线上bug，临时做替换\n处理
                     * case: http://crab.rutgers.edu/~dslun/biography.html
                     */
                    text = text.replace(/\n/g, '\u0020');

                    var item = {
                        sendback: type,
                        text: text
                    };

                    strLen += text;

                    if (strLen.length <= maxTextLength) {
                        // todo
                    } else {
                        texts.push(items);
                        strLen = '';
                        items = [];
                    }

                    // 增加 key： id
                    item.id = this.getid();
                    items.push(item);
                },

                /**
                 * [text 文本类型]
                 * @param  {[type]} type [description]
                 * @param  {[type]} node [description]
                 * @return {[type]}      [description]
                 */
                text: function (type, node) {
                    // textContent IE9+
                    var text = (node.textContent || node.nodeValue || '').trim(),
                        parent = node.parentNode,
                        len = parent ? parent.childNodes.length : 0,
                        font, parent;

                    if (!text) return;

                    this.push(text, type);
                    if (len > 1) {
                        font = document.createElement('font');
                        font.innerHTML = text;
                        parent.replaceChild(font, node);
                        this.setAttr(font, type);
                    } else {
                        this.setAttr(parent, type);
                    }
                },

                /**
                 * [wikitext wiki p tag get text]
                 * @param  {[type]} type [description]
                 * @param  {[type]} node [description]
                 * @return {[type]}      [description]
                 */
                wikitext: function (type, node) {
                    // textContent IE9+
                    var text = (node.innerText || node.textContent || '').trim();

                    if (!text) return;

                    this.push(text, type);
                    this.setAttr(node, type);
                },

                /**
                 * [attr set Attribute]
                 * @param  {[type]} attrName [description]
                 * @param  {[type]} node     [description]
                 * @return {[type]}          [description]
                 */
                attr: function (attrName, node) {
                    var text = (node.getAttribute(attrName) || '').trim();
                    if (!text) return;
                    this.push(text, attrName);
                    this.setAttr(node, attrName);
                }
            };

            /* Wiki Page Filter */
            var isWikiPage = /wikipedia\.org/.test(window.location.href);

            /**
             * [traverseNodes 递归遍历节点]
             * @param  {[type]} nodes [description]
             * @return {[type]}       [description]
             */
            function traverseNodes(nodes) {
                for (var i = 0, ilen = nodes.length; i < ilen; i++) {
                    var node = nodes[i],
                        nodeType = node.nodeType,
                        tagName, className, id;

                    if (nodeType == 1) {
                        tagName = node.tagName && node.tagName.toLowerCase();

                        /**
                         * 代码块支持，特殊站点过滤：
                         * http://www.w3schools.com
                         */
                        className = node.className;
                        if (className && /notranslate/g.test(className)) {
                            continue;
                        }
                        id = node.id;
                        if (id && /translate-sg-notice/.test(id)) {
                            continue;
                        }

                        switch (tagName) {
                            case 'meta':
                            case 'link':
                            case 'style':
                            case 'script':
                            case 'br':
                            case 'pre':
                            case 'code':
                            // case 'title':
                            case 'noscript':
                                continue;
                                break;
                            case 'li':
                            case 'dd':
                                var _parent = node.parentNode,
                                    _closest = _parent && _parent.parentNode;

                                if (isWikiPage && /mf-section/.test(_closest && _closest.className)) {
                                    typeList['wikitext']('text', node);
                                    continue;
                                }
                                break;
                            case 'p':
                                var _parent = node.parentNode;
                                if (isWikiPage && /mf-section/.test(_parent && _parent.className)) {
                                    typeList['wikitext']('text', node);
                                    continue;
                                }
                                break;
                            case 'iframe':
                                node.setAttribute('scrolling', 'yes');
                                continue;
                                break;
                            case 'input':
                                switch (node.type) {
                                    case 'hidden':
                                    case 'checkbox':
                                    case 'radio':
                                    case 'range':
                                        continue;
                                        break;
                                    case 'submit':
                                    case 'reset':
                                    case 'button':
                                    case 'image':
                                        typeList['attr']('value', node);
                                        break;
                                    case 'url':
                                    case 'tel':
                                    case 'number':
                                    case 'time':
                                    case 'date':
                                    case 'file':
                                    case 'email':
                                        typeList['attr']('placeholder', node);
                                        break;
                                    default:
                                        typeList['attr']('placeholder', node);
                                        typeList['attr']('value', node);
                                }
                                break;
                            case 'textarea':
                                typeList['attr']('placeholder', node);
                                break;
                            case 'img':
                                typeList['attr']('alt', node);
                                typeList['attr']('title', node);
                                break;
                            case 'a':
                            case 'area':
                                var href = node.getAttribute('href');
                                if (href && !/^\s*(?:javascript:|#)\s*/g.test(href)) {
                                    me.rewriteLink(node);
                                }
                                typeList['attr']('title', node);
                                break;
                            case 'form':
                                me.rewriteForm(node);
                                break;
                        }

                        if (node.childNodes && node.childNodes.length) {
                            traverseNodes(node.childNodes);
                            continue;
                        }
                    } else if (nodeType == 3) {
                        typeList['text']('text', node);
                    }
                }
            }

            traverseNodes(doc.documentElement.childNodes);

            // 最后一组入栈
            if (items.length) {
                texts.push(items);
            }

            me.texts = texts;

            me.getTextEndTime = Date.now();

            me.translateNodeText(texts, doc);


            return texts;
        },

        /**
         * [translateNodeText 翻译文本]
         * @param  {[type]} textsArr [description]
         * @param  {[type]} doc      [description]
         * @return {[type]}          [description]
         */
        translateNodeText: function (textsArr, doc) {
            var me = this,
                doc = doc || document,
                opts = me.options,
                from = opts.from,
                to = opts.to,
                beforeTranslate = opts.beforeTranslate,
                success = opts.success,
                error = opts.error,
                complete = opts.complete,
                textsArr = textsArr || [],
                params,
                isFun = function (fun) {
                    return 'function' === typeof fun;
                };

            if (isFun(beforeTranslate) && false === beforeTranslate.call(this, textsArr)) {
                return;
            }

            var resultArr = [];

            textsArr.length && textsArr.forEach(function (items, index) {
                params = {
                    uuid: me.uuid(),
                    from_lang: from,
                    to_lang: to,
                    sendback: '' + index,
                    trans_frag: items
                };

                resultArr.push({
                    url: opts.url,
                    method: 'post',
                    data: params
                });
            });

            /* App Callback */
            if (isFun(opts.appCallback)) {
                opts.appCallback(resultArr);
            }
        },

        /**
         * [getRand 获取随机数]
         * @return {[type]} [description]
         */
        getRand: function () {
            return (+new Date()) + '.r' + Math.floor(Math.random() * 1000);
        },

        /**
         * [reporter 日志上报接口]
         * @param  {[type]} paramsObj [description]
         * @return {[type]}           [description]
         */
        reporter: function (paramsObj) {
            var me = this,
                params = [],
                d = "memory_log_" + me.getRand(),
                img = window[d] = new Image(),
                url = '//pb.sogou.com/' + (paramsObj['pbtype'] || 'pv') + '.gif';

            delete paramsObj['pbtype'];

            paramsObj.uigs_productid = paramsObj.uigs_productid || 'wapapp';
            paramsObj.type = paramsObj.type || 'eng_fulltrans';

            for (var key in paramsObj) {
                if (paramsObj.hasOwnProperty(key)) {
                    var val = paramsObj[key];
                    !('undefined' === typeof val || '' === val) && params.push(key + '=' + window.encodeURIComponent(val));
                }
            }

            params.push('_t=' + me.getRand());
            url = url.indexOf('?') > 0 ? url : (url + '?');
            url = /\?.+[^&]$/.test(url) ? (url + '&') : url;
            url += params.join('&');

            img.onload = img.onerror = img.onabort = function () {
                img.onload = img.onerror = img.onabort = null;
                img = window.d = null;
            };
            img.src = url;
        }
    };

    if ('function' === typeof window.SogouTranslator) return;

    window.SogouTranslator = SogouTranslator;


    /**
     * transweb(data, responseCallback)
     *
     * data: 前端请求文本
     * responseCallback: app响应callback
     */
    var sogouTranslatorInstance = new SogouTranslator({
        url: 'http://translate.sogoucdn.com/commontranslate',
        from: 'en',
        to: 'zh-CHS',
        maxTextLength: 1000,
        appCallback: function (params) {
            var transweb = window.JSInvoker && 'function' === typeof window.JSInvoker.transWeb,
                initType = sogouTranslatorInstance._initType;
            transweb && window.JSInvoker.transWeb(JSON.stringify(params), "window.sogouTranslatorAppNodeText");
        }
    });

    /**
     * APP response
     * @function {function name}
     * @param  {Object} response {description}
     * @return {type} {description}
     */
    window.sogouTranslatorAppNodeText = function (response) {
        var me = sogouTranslatorInstance,
            responseStatus = response.status,
            responseData = response.data,
            textsArr = me.texts,
            items,
            destTexts, index, id, node, tagName, item, text, attrType,
            doc = document;

        if (responseStatus === 0) {
            if (responseData && responseData.status >= 0) {
                index = +responseData.sendback;
                items = textsArr ? textsArr[index] : '';
                destTexts = responseData.trans_result || [];

                if (destTexts.length !== items.length) {
                    console.log('srcLen != destLen');
                }

                for (var i = 0, ilen = destTexts.length; i < ilen; i++) {
                    item = destTexts[i];
                    text = item && item.trans_text;
                    id = me.extendAttrValue(item.id);
                    attrType = item.sendback;
                    node = doc.querySelector && doc.querySelector('[' + me.extendAttrName(attrType) + '="' + id + '"]');

                    // console.log(id, node, item.success, attrType == 'text' , node.hasAttribute(attrType));
                    if (node && item && item.success) {
                        if (attrType == 'text') {
                            node.innerHTML = text;
                        } else {
                            if (node.hasAttribute(attrType)) {
                                node.setAttribute(attrType, text);
                            }
                        }
                    }
                }
                //isFun(success) && success.call(me, data);
            } else {
                //isFun(error) && error.call(me, 'error', data);
            }
        } else if (responseStatus === 1) {
            //isFun(error) && error.call(me, 'error', data);
        }
    };

    /**
     * APP view origin texts
     */
    window.sogouTranslatorAppNodeOriginText = function () {
        var me = sogouTranslatorInstance,
            originTextsArr = me.texts || [];

        originTextsArr.forEach(function (items, index) {
            items.forEach(function (item, i) {
                var text, id, attrType, node;

                text = item.text;
                id = me.extendAttrValue(item.id);
                attrType = item.sendback;
                node = document.querySelector && document.querySelector('[' + me.extendAttrName(attrType) + '="' + id + '"]');

                if (node && item) {
                    if (attrType == 'text') {
                        node.innerHTML = text;
                    } else {
                        if (node.hasAttribute(attrType)) {
                            node.setAttribute(attrType, text);
                        }
                    }
                }
            })
        });
    }

    /**
     * APP init 接口
     */
    window.sogouTranslatorAppInit = function() {
        sogouTranslatorInstance.__initType = 1;
        sogouTranslatorInstance.init();
    };

    /**
     * APP 获取init type 接口, init 执行后调用
     */
    window.sogouTranslatorAppGetInitType = function() {
        return sogouTranslatorInstance.__initType || 0;
    }

    /**
     * 是否有 iframe 标识
     */
    window.sogouTranslatorAppHasIframe = function() {
        return !!document.body.querySelector('iframe');
    }

    /**
     * 客户端 init 提取文本供app侧做语种检测判断是否出底部悬浮翻译框
     */
    sogouTranslatorInstance.__initType = 0;
    sogouTranslatorInstance.init();
})(window);
